from typing import TYPE_CHECKING, Callable
from airflow.sdk.bases.decorator import task_decorator_factory
from airflow.operators.python import PythonVirtualenvOperator
from airflow.decorators.base import DecoratedOperator

if TYPE_CHECKING:
    from airflow.sdk.bases.decorator import TaskDecorator


class ClusterOperator(PythonVirtualenvOperator):
    def __init__(self, **kwargs):
        requirements = [
            "coiled",
            "dask[complete]",
            "tornado==6.4.2",
            "pandas==2.1.4",
            "numpy==1.26.4",
            "geopandas",
            "dask_geopandas",
            "s3fs",
            "pyarrow",
            "matplotlib",
        ]
        index_urls = ["https://pypi.org/simple"]
        super().__init__(requirements=requirements, index_urls=index_urls, **kwargs)

    def execute(self, context):
        super().execute(context=context)


class _ClusterDecoratedOperator(DecoratedOperator, ClusterOperator):
    custom_operator_name = "@task.coiled"


def coiled_task(
    callable: Callable | None = None, multiple_outputs: Callable | None = None, **kwargs
) -> "TaskDecorator":
    return task_decorator_factory(
        python_callable=callable,
        multiple_outputs=multiple_outputs,
        decorated_operator_class=_ClusterDecoratedOperator,
        **kwargs,
    )
